// Importing the Random class for shuffling the magic square
import java.util.Random;

public class MagicSquare {
    private int size;
    // Using a 2-D array as advised
    private int[][] square;

	// Class to store the square matrix (as advised)
    public MagicSquare(int size) {
        this.size = size;
        this.square = new int[size][size];
        generateMagicSquare();
    }
	
	// Generating a magic square using the given algorithm
	// Adjusting where necessary (e.g - 1-based indexing to 0-based)
    private void generateMagicSquare() {
        int x = 0, y = size / 2;
        for (int i = 1; i <= size * size; i++) {
            square[x][y] = i;
            int newX = (x - 1 + size) % size;
            int newY = (y - 1 + size) % size;

            if (square[newX][newY] == 0) {
                x = newX;
                y = newY;
            } else {
                x = (x + 1) % size;
            }
        }
    }
	
    public void shuffleSquare() {
        Random rand = new Random();
        for (int i = 0; i < size * size; i++) { // Repeat for n^2 (size^2) times
            int row = rand.nextInt(size); // Choosing a random elt
            int col = rand.nextInt(size); // Choosing a random elt
            // Swapping w/ random neighbor
            // U, D, L, R corresponds to 0,1,2,3
            int direction = rand.nextInt(4); // Swapping w/ random neighbor
            // Shuffling the magic square simulates what the player does, just with randomized values
            // This is why we call swapElement() in shuffleSquare() with true (swap happens during shuffling)
            swapElement(row, col, direction, true);
        }
    }

	// Swaps elts in the magic square to shuffle them
	// Does row/column shifts in the even that the swap moves the elt outside the square matrix (wrap around)
    public boolean swapElement(int row, int col, int direction, boolean isShuffling) {

    if (direction == 0) { // Up move
        if (row == 0) {  // First row → shift column up
            int firstVal = square[0][col];
            for (int i = 0; i < size - 1; i++) {
                square[i][col] = square[i + 1][col];
            }
            square[size - 1][col] = firstVal;
        } else {  // Just swap up with adjacent element
            int temp = square[row][col];
            square[row][col] = square[row - 1][col];
            square[row - 1][col] = temp;
        }

    } else if (direction == 1) { // Down move
        if (row == size - 1) {  // Last row → shift column down
            int lastVal = square[size - 1][col];
            for (int i = size - 1; i > 0; i--) {
                square[i][col] = square[i - 1][col];
            }
            square[0][col] = lastVal;
        } else {  // Just swap down with adjacent element
            int temp = square[row][col];
            square[row][col] = square[row + 1][col];
            square[row + 1][col] = temp;
        }

    } else if (direction == 2) { // Left move
        if (col == 0) {  // First column → shift entire row left to wrap around
            int firstVal = square[row][0];
            for (int j = 0; j < size - 1; j++) {
                square[row][j] = square[row][j + 1];
            }
            square[row][size - 1] = firstVal;
        } else {  // Just swap left with adjacent element
            int temp = square[row][col];
            square[row][col] = square[row][col - 1];
            square[row][col - 1] = temp;
        }

    } else if (direction == 3) { // Right move
        if (col == size - 1) {  // Last column → perform a row shift
            int lastVal = square[row][size - 1];
            for (int j = size - 1; j > 0; j--) {
                square[row][j] = square[row][j - 1];
            }
            square[row][0] = lastVal;
        } else {  // Just swap right with adjacent element
            int temp = square[row][col];
            square[row][col] = square[row][col + 1];
            square[row][col + 1] = temp;
        }

    } else {
        return false; // Invalid direction
    }

    return true; // Successful swap
}

    public boolean isMagicSquare() {
        int magicSum = size * (size * size + 1) / 2;

        for (int i = 0; i < size; i++) {
        // Reset rowSum and colSum at the start of each iteration
            int rowSum = 0, colSum = 0;
            for (int j = 0; j < size; j++) {
            	
                rowSum += square[i][j];
                // Switch i and j since Java uses row-major order
                colSum += square[j][i];
            }
            if (rowSum != magicSum || colSum != magicSum) return false;
        }

		// Need a separate for loop to iterate over the diagonals
        int diag1 = 0, diag2 = 0;
        for (int i = 0; i < size; i++) {
            diag1 += square[i][i];
            diag2 += square[i][size - 1 - i];
        }
        // This will return true if and only if both diagonals match the magic sum
        return diag1 == magicSum && diag2 == magicSum;
    }

    public void display() {
    // Iterate over each row 
        for (int[] row : square) {
        // Iterate over each number in the current row (since row is an array, an iterable)
            for (int num : row) {
            	// Print 3 spaces for the number including digits and padding
            	// This makes the grid look even
                System.out.printf("%3d ", num);
            }
            // After printing a row, move to the next line
            System.out.println();
        }
    }
}